<?php

namespace Drupal\overview_builder;

use Drupal\Component\Plugin\PluginBase;
use Drupal\Core\Cache\Cache;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\FormBuilderInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\RequestStack;

/**
 * Class OverviewBuilderBase.
 *
 * New overview builders should extend this class.
 */
abstract class OverviewBuilderBase extends PluginBase implements OverviewBuilderInterface, ContainerFactoryPluginInterface {

  /**
   * Nr of items shown per page.
   */
  protected const ITEMS_PER_PAGE = 10;

  /**
   * The current request.
   *
   * @var \Symfony\Component\HttpFoundation\Request|null
   */
  protected $currentRequest;

  /**
   * The entity storage.
   *
   * @var \Drupal\Core\Entity\EntityStorageInterface
   */
  protected $entityStorage;

  /**
   * The entity view builder.
   *
   * @var \Drupal\Core\Entity\EntityViewBuilderInterface
   */
  protected $entityViewBuilder;

  /**
   * The form builder.
   *
   * @var \Drupal\Core\Form\FormBuilderInterface
   */
  protected $formBuilder;

  /**
   * OverviewBuilderBase constructor.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   *   The entity type manager.
   * @param \Drupal\Core\Form\FormBuilderInterface $formBuilder
   *   The form builder.
   * @param \Symfony\Component\HttpFoundation\RequestStack $requestStack
   *   The request stack.
   *
   * @throws \Exception
   *   When no entity_type is configured in the annotation, throw an error.
   */
  public function __construct(array $configuration, string $plugin_id, $plugin_definition, EntityTypeManagerInterface $entityTypeManager, FormBuilderInterface $formBuilder, RequestStack $requestStack) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);

    if ($this->getEntityType() === NULL) {
      // When no entity_type is configured in the annotation, throw an error.
      throw new \Exception(sprintf('No entity_type configured in %s.', get_class($this)));
    }

    $this->currentRequest = $requestStack->getCurrentRequest();
    $this->entityStorage = $entityTypeManager->getStorage($this->getEntityType());
    $this->entityViewBuilder = $entityTypeManager->getViewBuilder($this->getEntityType());
    $this->formBuilder = $formBuilder;
  }

  /**
   * Get the entity type that is shown in the overview.
   *
   * @return string|null
   *   The entity type.
   */
  protected function getEntityType() {
    return $this->pluginDefinition['entity_type'] ?? NULL;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('entity_type.manager'),
      $container->get('form_builder'),
      $container->get('request_stack')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function build(): array {
    return [
      '#theme' => 'overview_builder__' . $this->getPluginId(),
      '#filters' => $this->buildFilters(),
      '#items' => $this->buildItems(),
      '#pager' => $this->buildPager(),
    ];
  }

  /**
   * Build the filters. Returns a form if there are filters.
   *
   * @return array|null
   *   The filters form.
   */
  protected function buildFilters(): ?array {
    return NULL;
  }

  /**
   * Builds the items in the overview.
   *
   * @return array
   *   An array containing the items shown in the overview.
   */
  protected function buildItems(): array {
    $entities = $this->getListEntities();

    $items = [];
    if ($entities) {
      foreach ($entities as $entity) {
        $items[] = $this->entityViewBuilder->view($entity, $this->getViewMode());
      }
    }

    return $items;
  }

  /**
   * Build a list of entities shown in the overview.
   *
   * @return \Drupal\Core\Entity\EntityInterface[]
   *   The entities shown in the overview.
   */
  protected function getListEntities(): array {
    return [];
  }

  /**
   * Get the configured view mode from the annotation.
   *
   * @return string
   *   The view mode used in the overview.
   */
  protected function getViewMode(): string {
    return $this->pluginDefinition['view_mode'] ?? 'teaser';
  }

  /**
   * Build the pager.
   *
   * @return array
   *   A renderable array containing the pager.
   */
  protected function buildPager(): array {
    return [
      '#type' => 'pager',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function getCacheContexts() {
    return [
      'url.query_args',
      'languages',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function getCacheMaxAge() {
    return Cache::PERMANENT;
  }

  /**
   * {@inheritdoc}
   */
  public function getCacheTags() {
    return [
      $this->getEntityType() . '_list',
    ];
  }

}
